@@ -1,27 +1,30 @@
-from __future__ import (absolute_import, print_function, division)
-
-import traceback
-import sys
 import socket
-import six
+import sys
+import traceback
 
-from netlib import tcp
-from netlib.exceptions import TcpException
-from netlib.http.http1 import assemble_response
-from ..exceptions import ProtocolException, ServerException, ClientHandshakeException
-from ..protocol import Kill
-from ..models import ClientConnection, make_error_response
-from .modes import HttpUpstreamProxy, HttpProxy, ReverseProxy, TransparentProxy, Socks5Proxy
-from .root_context import RootContext, Log
+from mitmproxy import exceptions
+from mitmproxy import connections
+from mitmproxy import http
+from mitmproxy import log
+from mitmproxy import platform
+from mitmproxy.proxy import ProxyConfig
+from mitmproxy.proxy import modes
+from mitmproxy.proxy import root_context
+from mitmproxy.net import tcp
+from mitmproxy.net.http import http1
 
 
 class DummyServer:
     bound = False
 
-    def __init__(self, config):
+    def __init__(self, config=None):
         self.config = config
+        self.address = "dummy"
 
-    def start_slave(self, *args):
+    def set_channel(self, channel):
+        pass
+
+    def serve_forever(self):
         pass
 
     def shutdown(self):
@@ -32,25 +35,23 @@ class ProxyServer(tcp.TCPServer):
     allow_reuse_address = True
     bound = True
 
-    def __init__(self, config):
+    def __init__(self, config: ProxyConfig):
         """
-            Raises ProxyServerError if there's a startup problem.
+            Raises ServerException if there's a startup problem.
         """
         self.config = config
         try:
-            super(ProxyServer, self).__init__((config.host, config.port))
+            super().__init__(
+                (config.options.listen_host, config.options.listen_port)
+            )
+            if config.options.mode == "transparent":
+                platform.init_transparent_mode()
         except socket.error as e:
-            six.reraise(
-                ServerException,
-                ServerException('Error starting proxy server: ' + repr(e)),
-                sys.exc_info()[2]
+            raise exceptions.ServerException(
+                'Error starting proxy server: ' + repr(e)
             )
         self.channel = None
 
-    def start_slave(self, klass, channel):
-        slave = klass(channel, self)
-        slave.start()
-
     def set_channel(self, channel):
         self.channel = channel
 
@@ -64,12 +65,12 @@ class ProxyServer(tcp.TCPServer):
         h.handle()
 
 
-class ConnectionHandler(object):
+class ConnectionHandler:
 
     def __init__(self, client_conn, client_address, config, channel):
         self.config = config
         """@type: mitmproxy.proxy.config.ProxyConfig"""
-        self.client_conn = ClientConnection(
+        self.client_conn = connections.ClientConnection(
             client_conn,
             client_address,
             None)
@@ -78,33 +79,33 @@ class ConnectionHandler(object):
         """@type: mitmproxy.controller.Channel"""
 
     def _create_root_layer(self):
-        root_context = RootContext(
+        root_ctx = root_context.RootContext(
             self.client_conn,
             self.config,
             self.channel
         )
 
-        mode = self.config.mode
+        mode = self.config.options.mode
         if mode == "upstream":
-            return HttpUpstreamProxy(
-                root_context,
+            return modes.HttpUpstreamProxy(
+                root_ctx,
                 self.config.upstream_server.address
             )
         elif mode == "transparent":
-            return TransparentProxy(root_context)
+            return modes.TransparentProxy(root_ctx)
         elif mode == "reverse":
             server_tls = self.config.upstream_server.scheme == "https"
-            return ReverseProxy(
-                root_context,
+            return modes.ReverseProxy(
+                root_ctx,
                 self.config.upstream_server.address,
                 server_tls
             )
         elif mode == "socks5":
-            return Socks5Proxy(root_context)
+            return modes.Socks5Proxy(root_ctx)
         elif mode == "regular":
-            return HttpProxy(root_context)
+            return modes.HttpProxy(root_ctx)
         elif callable(mode):  # pragma: no cover
-            return mode(root_context)
+            return mode(root_ctx)
         else:  # pragma: no cover
             raise ValueError("Unknown proxy mode: %s" % mode)
 
@@ -112,35 +113,34 @@ class ConnectionHandler(object):
         self.log("clientconnect", "info")
 
         root_layer = self._create_root_layer()
-        root_layer = self.channel.ask("clientconnect", root_layer)
-        if root_layer == Kill:
-            def root_layer():
-                raise Kill()
 
         try:
+            root_layer = self.channel.ask("clientconnect", root_layer)
             root_layer()
-        except Kill:
+        except exceptions.Kill:
             self.log("Connection killed", "info")
-        except ProtocolException as e:
-
-            if isinstance(e, ClientHandshakeException):
+        except exceptions.ProtocolException as e:
+            if isinstance(e, exceptions.ClientHandshakeException):
                 self.log(
                     "Client Handshake failed. "
                     "The client may not trust the proxy's certificate for {}.".format(e.server),
-                    "error"
+                    "warn"
                 )
                 self.log(repr(e), "debug")
+            elif isinstance(e, exceptions.InvalidServerCertificate):
+                self.log(str(e), "warn")
+                self.log("Invalid certificate, closing connection. Pass --insecure to disable validation.", "warn")
             else:
-                self.log(repr(e), "info")
+                self.log(str(e), "warn")
 
-                self.log(traceback.format_exc(), "debug")
+                self.log(repr(e), "debug")
             # If an error propagates to the topmost level,
             # we send an HTTP error response, which is both
             # understandable by HTTP clients and humans.
             try:
-                error_response = make_error_response(502, repr(e))
-                self.client_conn.send(assemble_response(error_response))
-            except TcpException:
+                error_response = http.make_error_response(502, repr(e))
+                self.client_conn.send(http1.assemble_response(error_response))
+            except exceptions.TcpException:
                 pass
         except Exception:
             self.log(traceback.format_exc(), "error")
@@ -154,4 +154,4 @@ class ConnectionHandler(object):
 
     def log(self, msg, level):
         msg = "{}: {}".format(repr(self.client_conn.address), msg)
-        self.channel.tell("log", Log(msg, level))
+        self.channel.tell("log", log.LogEntry(msg, level))
