@@ -4,9 +4,9 @@ import os
 import os.path
 import re
 
-from netlib import tcp
-from netlib import human
-from netlib import version
+from mitmproxy.net import tcp
+from mitmproxy.utils import human
+from mitmproxy import version
 from . import pathod
 
 
